/** \file
 * Библиотека криптографических алгоритмов ГОСТ.
 */

#ifndef GOSTCRYPTO_H
#define GOSTCRYPTO_H

#include <stdint.h>
#include <stdbool.h>

/**
 * Размер ключа симметричного шифра (в байтах)
 */
#define SYMMETRIC_KEY_SIZE 32

/**
 * Размер синхропосылки #KuznyechikMgm (в байтах)
 */
#define KUZNYECHIK_MGM_NONCE_SIZE 16

/**
 * Размер имитовставки #KuznyechikMgm (в байтах)
 */
#define KUZNYECHIK_MGM_TAG_SIZE 16

/**
 * Выравнивание контекста #KuznyechikMgm (в байтах)
 */
#define KUZNYECHIK_MGM_ALIGNMENT 1

/**
 * Размер контекста #KuznyechikMgm (в байтах)
 */
#define KUZNYECHIK_MGM_SIZE 160

/**
 * Размер синхропосылки #KuznyechikCbc (в байтах)
 */
#define KUZNYECHIK_CBC_NONCE_SIZE 16

/**
 * Выравнивание #KuznyechikCbc (в байтах)
 */
#define KUZNYECHIK_CBC_ALIGNMENT 1

/**
 * Размер контекста #KuznyechikCbc (в байтах)
 */
#define KUZNYECHIK_CBC_SIZE 177

/**
 * Размер синхропосылки #MagmaCbc (в байтах)
 */
#define MAGMA_CBC_NONCE_SIZE 8

/**
 * Выравнивание #MagmaCbc (в байтах)
 */
#define MAGMA_CBC_ALIGNMENT 4

/**
 * Размер контекста #MagmaCbc (в байтах)
 */
#define MAGMA_CBC_SIZE 44

/**
 * Размер синхропосылки #KuznyechikCfb (в байтах)
 */
#define KUZNYECHIK_CFB_NONCE_SIZE 16

/**
 * Выравнивание #KuznyechikCfb (в байтах)
 */
#define KUZNYECHIK_CFB_ALIGNMENT 1

/**
 * Размер контекста #KuznyechikCfb (в байтах)
 */
#define KUZNYECHIK_CFB_SIZE 177

/**
 * Размер синхропосылки #MagmaCfb (в байтах)
 */
#define MAGMA_CFB_NONCE_SIZE 8

/**
 * Выравнивание #MagmaCfb (в байтах)
 */
#define MAGMA_CFB_ALIGNMENT 4

/**
 * Размер контекста #MagmaCfb (в байтах)
 */
#define MAGMA_CFB_SIZE 44

/**
 * Размер блока шифра #Kuznyechik (в байтах)
 */
#define KUZNYECHIK_BLOCK_SIZE 16

/**
 * Выравнивание #Kuznyechik (в байтах)
 */
#define KUZNYECHIK_ALIGNMENT 1

/**
 * Размер контекста #Kuznyechik (в байтах)
 */
#define KUZNYECHIK_SIZE 160

/**
 * Размер блока #Magma (в байтах)
 */
#define MAGMA_BLOCK_SIZE 8

/**
 * Выравнивание #Magma (в байтах)
 */
#define MAGMA_ALIGNMENT 4

/**
 * Размер контекста #Magma (в байтах)
 */
#define MAGMA_SIZE 32

/**
 * Размер синхропосылки #KuznyechikCtr (в байтах)
 */
#define KUZNYECHIK_CTR_NONCE_SIZE 8

/**
 * Выравнивание #KuznyechikCtr (в байтах)
 */
#define KUZNYECHIK_CTR_ALIGNMENT 8

/**
 * Размер контекста #KuznyechikCtr (в байтах)
 */
#define KUZNYECHIK_CTR_SIZE 200

/**
 * Размер синхропосылки #MagmaCtr (в байтах)
 */
#define MAGMA_CTR_NONCE_SIZE 4

/**
 * Выравнивание #MagmaCtr (в байтах)
 */
#define MAGMA_CTR_ALIGNMENT 4

/**
 * Размер контекста #MagmaCtr (в байтах)
 */
#define MAGMA_CTR_SIZE 52

/**
 * Выравнивание #KuznyechikEcb (в байтах)
 */
#define KUZNYECHIK_ECB_ALIGNMENT 1

/**
 * Размер контекста #KuznyechikEcb (в байтах)
 */
#define KUZNYECHIK_ECB_SIZE 160

/**
 * Выравнивание #MagmaEcb (в байтах)
 */
#define MAGMA_ECB_ALIGNMENT 4

/**
 * Размер контекста #MagmaEcb (в байтах)
 */
#define MAGMA_ECB_SIZE 32

/**
 * Длина закрытого ключа и хэша сообщения кривых #Ec256 (в байтах)
 */
#define EC_256_WIDTH 32

/**
 * Длина сериализованного открытого ключа и подписи кривых #Ec256 (в байтах)
 */
#define EC_256_DOUBLE_WIDTH 64

/**
 * Выравнивание закрытого ключа кривых #Ec256 (в байтах)
 */
#define EC_256_PRIVATE_KEY_ALIGNMENT 8

/**
 * Длина несериализованного открытого ключа кривых #Ec256 (в байтах)
 */
#define EC_256_PUBLIC_KEY_SIZE 96

/**
 * Выравнивание несериализованного открытого ключа кривых #Ec256 (в байтах)
 */
#define EC_256_PUBLIC_KEY_ALIGNMENT 8

/**
 * Идентификатор кривой `id-tc26-gost-3410-12-512-paramSetA`, заданной в Р 1323565.1.024-2019.
 */
#define GOST3410_2012_512A 643712121

/**
 * Идентификатор кривой `id-tc26-gost-3410-12-512-paramSetB`, заданной в Р 1323565.1.024-2019.
 */
#define GOST3410_2012_512B 643712122

/**
 * Идентификатор кривой `id-tc26-gost-3410-12-512-paramSetC`, заданной в Р 1323565.1.024-2019.
 */
#define GOST3410_2012_512C 643712123

/**
 * Длина закрытого ключа и хэша сообщения кривых #Ec512 (в байтах)
 */
#define EC_512_WIDTH 64

/**
 * Длина сериализованного открытого ключа и подписи кривых #Ec512 (в байтах)
 */
#define EC_512_DOUBLE_WIDTH 128

/**
 * Выравнивание закрытого ключа кривых #Ec512 (в байтах)
 */
#define EC_512_PRIVATE_KEY_ALIGNMENT 8

/**
 * Длина несериализованного открытого ключа кривых #Ec512 (в байтах)
 */
#define EC_512_PUBLIC_KEY_SIZE 192

/**
 * Выравнивание несериализованного открытого ключа кривых #Ec512 (в байтах)
 */
#define EC_512_PUBLIC_KEY_ALIGNMENT 8

/**
 * Некорректный идентификатор эллиптической кривой.
 */
#define EC_ERR_CURVE_ID -2

/**
 * Сериализованное представление открытого ключа некорректно.
 */
#define EC_ERR_PUBLIC_KEY_BYTES -3

/**
 * Сериализованное представление закрытого ключа некорректно.
 */
#define EC_ERR_PRIVATE_KEY_BYTES -4

/**
 * Некорректная подпись для данного хэша/сообщения и открытого ключа.
 */
#define EC_ERR_SIGNATURE -5

/**
 * Невозможно выполнить операцию с данными случайными числами.
 * Случайные числа не удовлетворяют условиям (ГОСТ 34.10-2012 и ГОСТ 34.10-2018).
 * Повторите операцию с другими случайными числами.
 */
#define EC_ERR_BAD_RAND_BYTES -6

/**
 * Нулевое значение UKM по модулю простого числа эллиптической кривой.
 */
#define EC_ERR_VKO_UKM -7

/**
 * Длина хэш-кода #Streebog256 (в байтах)
 */
#define STREEBOG256_RESULT_SIZE 32

/**
 * Выравнивание состояния #Streebog256 (в байтах)
 */
#define STREEBOG256_ALIGNMENT 8

/**
 * Размер состояния #Streebog256 (в байтах)
 */
#define STREEBOG256_SIZE 264

/**
 * Длина хэш-кода #Streebog512 (в байтах)
 */
#define STREEBOG512_RESULT_SIZE 64

/**
 * Выравнивание состояния #Streebog512 (в байтах)
 */
#define STREEBOG512_ALIGNMENT 8

/**
 * Размер состояния #Streebog512 (в байтах)
 */
#define STREEBOG512_SIZE 264

/**
 * Длина имитовставки #KuznyechikMac (в байтах)
 */
#define KUZNYECHIK_MAC_RESULT_SIZE 16

/**
 * Выравнивание состояния #KuznyechikMac (в байтах)
 */
#define KUZNYECHIK_MAC_ALIGNMENT 8

/**
 * Размер состояния #KuznyechikMac (в байтах)
 */
#define KUZNYECHIK_MAC_SIZE 216

/**
 * Длина имитовставки #MagmaMac (в байтах)
 */
#define MAGMA_MAC_RESULT_SIZE 8

/**
 * Выравнивание состояния #MagmaMac (в байтах)
 */
#define MAGMA_MAC_ALIGNMENT 8

/**
 * Размер состояния #MagmaMac (в байтах)
 */
#define MAGMA_MAC_SIZE 64

/**
 * Размер синхропосылки #KuznyechikOfb (в байтах)
 */
#define KUZNYECHIK_OFB_NONCE_SIZE 16

/**
 * Выравнивание #KuznyechikOfb (в байтах)
 */
#define KUZNYECHIK_OFB_ALIGNMENT 1

/**
 * Размер контекста #KuznyechikOfb (в байтах)
 */
#define KUZNYECHIK_OFB_SIZE 178

/**
 * Размер синхропосылки #MagmaOfb (в байтах)
 */
#define MAGMA_OFB_NONCE_SIZE 8

/**
 * Выравнивание #MagmaOfb (в байтах)
 */
#define MAGMA_OFB_ALIGNMENT 4

/**
 * Размер контекста #MagmaOfb (в байтах)
 */
#define MAGMA_OFB_SIZE 44

/**
 * Блочный шифр с длиной блока 128 бит в режиме работы блочных шифров, реализующим
 * аутентифицирующее шифрование с присоединёнными данными (AEAD). Определен в Р 1323565.1.026–2019.
 *
 * Набор функций для работы с MGM:
 *
 * - #kuznyechik_mgm_new
 * - #kuznyechik_mgm_encrypt
 * - #kuznyechik_mgm_decrypt
 * - #kuznyechik_mgm_drop
 *
 * Набор констант для работы с MGM:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_MGM_SIZE
 * - #KUZNYECHIK_MGM_ALIGNMENT
 * - #KUZNYECHIK_MGM_NONCE_SIZE
 * - #KUZNYECHIK_MGM_TAG_SIZE
 */
typedef uint8_t KuznyechikMgm[KUZNYECHIK_MGM_SIZE];

/**
 * Блочный шифр с длиной блока 128 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 16 байт. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cbc_new
 * - #kuznyechik_cbc_encrypt_blocks
 * - #kuznyechik_cbc_decrypt_blocks
 * - #kuznyechik_cbc_encrypt_pad
 * - #kuznyechik_cbc_decrypt_pad
 * - #kuznyechik_cbc_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_CBC_SIZE
 * - #KUZNYECHIK_CBC_ALIGNMENT
 * - #KUZNYECHIK_CBC_NONCE_SIZE
 * - #KUZNYECHIK_BLOCK_SIZE
 */
typedef uint8_t KuznyechikCbc[KUZNYECHIK_CBC_SIZE];

/**
 * Блочный шифр с длиной блока 64 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 8 байт. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cbc_new
 * - #magma_cbc_encrypt_blocks
 * - #magma_cbc_decrypt_blocks
 * - #magma_cbc_encrypt_pad
 * - #magma_cbc_decrypt_pad
 * - #magma_cbc_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #MAGMA_CBC_SIZE
 * - #MAGMA_CBC_ALIGNMENT
 * - #MAGMA_CBC_NONCE_SIZE
 * - #MAGMA_BLOCK_SIZE
 */
typedef uint8_t MagmaCbc[MAGMA_CBC_SIZE];

/**
 * Потоковый шифр с длиной блока 128 бит в режиме гаммирования с обратной связью по шифртексту
 * с размером синхропосылки 16 байт. Определен в ГОСТ 34.13-2015 и  ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cfb_new
 * - #kuznyechik_cfb_encrypt
 * - #kuznyechik_cfb_decrypt
 * - #kuznyechik_cfb_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_CFB_SIZE
 * - #KUZNYECHIK_CFB_ALIGNMENT
 * - #KUZNYECHIK_CFB_NONCE_SIZE
 */
typedef uint8_t KuznyechikCfb[KUZNYECHIK_CFB_SIZE];

/**
 * Потоковый шифр с длиной блока 64 бит в режиме гаммирования с обратной связью по шифртексту
 * с размером синхропосылки 8 байт. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cfb_new
 * - #magma_cfb_encrypt
 * - #magma_cfb_decrypt
 * - #magma_cfb_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #MAGMA_CFB_SIZE
 * - #MAGMA_CFB_ALIGNMENT
 * - #MAGMA_CFB_NONCE_SIZE
 */
typedef uint8_t MagmaCfb[MAGMA_CFB_SIZE];

/**
 * Блочный шифр с длиной блока 128 бит. Определён в ГОСТ Р 34.12-2015 (Кузнечик), ГОСТ 34.12-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_new
 * - #kuznyechik_encrypt_block
 * - #kuznyechik_decrypt_block
 * - #kuznyechik_encrypt_blocks
 * - #kuznyechik_decrypt_blocks
 * - #kuznyechik_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_SIZE
 * - #KUZNYECHIK_ALIGNMENT
 * - #KUZNYECHIK_BLOCK_SIZE
 */
typedef uint8_t Kuznyechik[KUZNYECHIK_SIZE];

/**
 * Блочный шифр с длиной блока 64 бит. Определён в ГОСТ Р 34.12-2015 (Магма), ГОСТ 34.12-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_new
 * - #magma_encrypt_block
 * - #magma_decrypt_block
 * - #magma_encrypt_blocks
 * - #magma_decrypt_blocks
 * - #magma_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #MAGMA_SIZE
 * - #MAGMA_ALIGNMENT
 * - #MAGMA_BLOCK_SIZE
 */
typedef uint8_t Magma[MAGMA_SIZE];

/**
 * Потоковый шифр с длиной блока 128 бит в режиме гаммирования. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ctr_new
 * - #kuznyechik_ctr_apply_keystream
 * - #kuznyechik_ctr_seek
 * - #kuznyechik_ctr_get_pos
 * - #kuznyechik_ctr_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_CTR_SIZE
 * - #KUZNYECHIK_CTR_ALIGNMENT
 * - #KUZNYECHIK_CTR_NONCE_SIZE
 */
typedef uint8_t KuznyechikCtr[KUZNYECHIK_CTR_SIZE];

/**
 * Потоковый шифр с длиной блока 64 бит в режиме гаммирования. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ctr_new
 * - #magma_ctr_apply_keystream
 * - #magma_ctr_seek
 * - #magma_ctr_get_pos
 * - #magma_ctr_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #MAGMA_CTR_SIZE
 * - #MAGMA_CTR_ALIGNMENT
 * - #MAGMA_CTR_NONCE_SIZE
 */
typedef uint8_t MagmaCtr[MAGMA_CTR_SIZE];

/**
 * Блочный шифр с длиной блока 128 бит в режиме простой замены. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ecb_new
 * - #kuznyechik_ecb_encrypt_blocks
 * - #kuznyechik_ecb_decrypt_blocks
 * - #kuznyechik_ecb_encrypt_pad
 * - #kuznyechik_ecb_decrypt_pad
 * - #kuznyechik_ecb_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_ECB_SIZE
 * - #KUZNYECHIK_ECB_ALIGNMENT
 * - #KUZNYECHIK_BLOCK_SIZE
 */
typedef uint8_t KuznyechikEcb[KUZNYECHIK_ECB_SIZE];

/**
 * Блочный шифр с длиной блока 64 бит в режиме простой замены. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ecb_new
 * - #magma_ecb_encrypt_blocks
 * - #magma_ecb_decrypt_blocks
 * - #magma_ecb_encrypt_pad
 * - #magma_ecb_decrypt_pad
 * - #magma_ecb_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #MAGMA_ECB_SIZE
 * - #MAGMA_ECB_ALIGNMENT
 * - #MAGMA_BLOCK_SIZE
 */
typedef uint8_t MagmaEcb[MAGMA_ECB_SIZE];

/**
 * Алгоритмы формирования, проверки электронной цифровой подписи
 * и выработки общих ключей. Определены в ГОСТ 34.10-2012 и ГОСТ 34.10-2018.
 *
 * Типы ключей для работы с эллиптическими кривыми:
 *
 * - #Ec256PrivateKey
 * - #Ec256PublicKey
 *
 * Набор функций для работы с эллиптическими кривыми:
 *
 * - #ec256_gen_private_key
 * - #ec256_get_public_key
 * - #ec256_private_key_from_bytes
 * - #ec256_private_key_to_bytes
 * - #ec256_public_key_from_bytes
 * - #ec256_public_key_to_bytes
 * - #ec256_dsa_sign
 * - #ec256_dsa_verify
 * - #ec256_dsa_sign_hash
 * - #ec256_dsa_verify_hash
 * - #ec256_vko256
 * - #ec256_private_key_drop
 *
 * Идентификаторы эллиптических кривых:
 *
 * - #GOST3410_2012_256A
 * - #GOST3410_2012_256B
 * - #GOST3410_2012_256C
 * - #GOST3410_2012_256D
 *
 * Набор констант для работы с эллиптическими кривыми:
 *
 * - #EC_256_WIDTH
 * - #EC_256_DOUBLE_WIDTH
 * - #EC_256_PRIVATE_KEY_ALIGNMENT
 * - #EC_256_PUBLIC_KEY_SIZE
 * - #EC_256_PUBLIC_KEY_ALIGNMENT
 */
typedef uintptr_t Ec256;

/**
 * Несериализованный закрытый ключ кривых #Ec256 длиной #EC_256_WIDTH и выравниванием #EC_256_PRIVATE_KEY_ALIGNMENT
 *
 * \warning Работа ключа одной кривой с идентификатором другой кривой приводит к неопределенному поведению.
 */
typedef uint8_t Ec256PrivateKey[EC_256_WIDTH];

/**
 * Несериализованный открытый ключ кривых #Ec256 длиной #EC_256_PUBLIC_KEY_SIZE и выравниванием #EC_256_PUBLIC_KEY_ALIGNMENT
 *
 * \warning Работа ключа одной кривой с идентификатором другой кривой приводит к неопределенному поведению.
 */
typedef uint8_t Ec256PublicKey[EC_256_PUBLIC_KEY_SIZE];

/**
 * Алгоритмы формирования, проверки электронной цифровой подписи
 * и выработки общих ключей. Определены в ГОСТ 34.10-2012 и ГОСТ 34.10-2018.
 *
 * Типы ключей для работы с эллиптическими кривыми:
 *
 * - #Ec512PrivateKey
 * - #Ec512PublicKey
 *
 * Набор функций для работы с эллиптическими кривыми:
 *
 * - #ec512_gen_private_key
 * - #ec512_get_public_key
 * - #ec512_private_key_from_bytes
 * - #ec512_private_key_to_bytes
 * - #ec512_public_key_from_bytes
 * - #ec512_public_key_to_bytes
 * - #ec512_dsa_sign
 * - #ec512_dsa_verify
 * - #ec512_dsa_sign_hash
 * - #ec512_dsa_verify_hash
 * - #ec512_vko256
 * - #ec512_vko512
 * - #ec512_private_key_drop
 *
 * Идентификаторы эллиптических кривых:
 *
 * - #GOST3410_2012_512A
 * - #GOST3410_2012_512B
 * - #GOST3410_2012_512C
 *
 * Набор констант для работы с эллиптическими кривыми:
 *
 * - #EC_512_WIDTH
 * - #EC_512_DOUBLE_WIDTH
 * - #EC_512_PRIVATE_KEY_ALIGNMENT
 * - #EC_512_PUBLIC_KEY_SIZE
 * - #EC_512_PUBLIC_KEY_ALIGNMENT
 */
typedef uintptr_t Ec512;

/**
 * Несериализованный закрытый ключ кривых #Ec512 длиной #EC_512_WIDTH и выравниванием #EC_512_PRIVATE_KEY_ALIGNMENT
 *
 * \warning Работа ключа одной кривой с идентификатором другой кривой приводит к неопределенному поведению.
 */
typedef uint8_t Ec512PrivateKey[EC_512_WIDTH];

/**
 * Несериализованный открытый ключ кривых #Ec512 длиной #EC_512_PUBLIC_KEY_SIZE и выравниванием #EC_512_PUBLIC_KEY_ALIGNMENT
 *
 * \warning Работа ключа одной кривой с идентификатором другой кривой приводит к неопределенному поведению.
 */
typedef uint8_t Ec512PublicKey[EC_512_PUBLIC_KEY_SIZE];

/**
 * Функция хэширования с длиной хэш-кода 256 бит. Определена в ГОСТ Р 34.11-2012 (Стрибог) и ГОСТ 34.11-2018.
 *
 * Набор функций для работы с функцией хэширования:
 *
 * - #streebog256_new
 * - #streebog256_digest
 * - #streebog256_update
 * - #streebog256_finalize
 * - #streebog256_finalize_reset
 * - #streebog256_reset
 * - #streebog256_drop
 *
 * Набор констант для работы с функцией хэширования:
 *
 * - #STREEBOG256_SIZE
 * - #STREEBOG256_ALIGNMENT
 * - #STREEBOG256_RESULT_SIZE
 */
typedef uint8_t Streebog256[STREEBOG256_SIZE];

/**
 * Функция хэширования с длиной хэш-кода 512 бит. Определена в ГОСТ Р 34.11-2012 (Стрибог) и ГОСТ 34.11-2018.
 *
 * Набор функций для работы с функцией хэширования:
 *
 * - #streebog512_new
 * - #streebog512_digest
 * - #streebog512_update
 * - #streebog512_finalize
 * - #streebog512_finalize_reset
 * - #streebog512_reset
 * - #streebog512_drop
 *
 * Набор констант для работы с функцией хэширования:
 *
 * - #STREEBOG512_SIZE
 * - #STREEBOG512_ALIGNMENT
 * - #STREEBOG512_RESULT_SIZE
 */
typedef uint8_t Streebog512[STREEBOG512_SIZE];

/**
 * Режим выработки имитовставки с использованием шифра с длиной блока 128 бит. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #kuznyechik_mac_new
 * - #kuznyechik_mac_update
 * - #kuznyechik_mac_finalize
 * - #kuznyechik_mac_finalize_reset
 * - #kuznyechik_mac_verify
 * - #kuznyechik_mac_reset
 * - #kuznyechik_mac_drop
 *
 * Набор констант для работы с режимом выработки имитовставки:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_MAC_SIZE
 * - #KUZNYECHIK_MAC_ALIGNMENT
 * - #KUZNYECHIK_MAC_RESULT_SIZE
 */
typedef uint8_t KuznyechikMac[KUZNYECHIK_MAC_SIZE];

/**
 * Режим выработки имитовставки с использованием шифра с длиной блока 64 бит. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #magma_mac_new
 * - #magma_mac_update
 * - #magma_mac_finalize
 * - #magma_mac_finalize_reset
 * - #magma_mac_verify
 * - #magma_mac_reset
 * - #magma_mac_drop
 *
 * Набор констант для работы с режимом выработки имитовставки:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #MAGMA_MAC_SIZE
 * - #MAGMA_MAC_ALIGNMENT
 * - #MAGMA_MAC_RESULT_SIZE
 */
typedef uint8_t MagmaMac[MAGMA_MAC_SIZE];

/**
 * Потоковый шифр с длиной блока 128 бит в режиме гаммирования с обратной связью по выходу
 * с размером синхропосылки 16 байт. Определен в ГОСТ Р 34.13-2015  и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ofb_new
 * - #kuznyechik_ofb_apply_keystream
 * - #kuznyechik_ofb_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #KUZNYECHIK_OFB_SIZE
 * - #KUZNYECHIK_OFB_ALIGNMENT
 * - #KUZNYECHIK_OFB_NONCE_SIZE
 */
typedef uint8_t KuznyechikOfb[KUZNYECHIK_OFB_SIZE];

/**
 * Потоковый шифр с длиной блока 64 бит в режиме гаммирования с обратной связью по выходу
 * с размером синхропосылки 8 байт. Определен в ГОСТ Р 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ofb_new
 * - #magma_ofb_apply_keystream
 * - #magma_ofb_drop
 *
 * Набор констант для работы с шифром:
 *
 * - #SYMMETRIC_KEY_SIZE
 * - #MAGMA_OFB_SIZE
 * - #MAGMA_OFB_ALIGNMENT
 * - #MAGMA_OFB_NONCE_SIZE
 */
typedef uint8_t MagmaOfb[MAGMA_OFB_SIZE];

/**
 * Идентификатор кривой `id-tc26-gost-3410-2012-256-paramSetA`, заданной в Р 1323565.1.024-2019.
 */
#define GOST3410_2012_256A 643712111

/**
 * Идентификатор кривой `id-tc26-gost-3410-2012-256-paramSetB`, заданной в Р 1323565.1.024-2019.
 */
#define GOST3410_2012_256B 643712112

/**
 * Идентификатор кривой `id-tc26-gost-3410-2012-256-paramSetC`, заданной в Р 1323565.1.024-2019.
 */
#define GOST3410_2012_256C 643712113

/**
 * Идентификатор кривой `id-tc26-gost-3410-2012-256-paramSetD`, заданной в Р 1323565.1.024-2019.
 */
#define GOST3410_2012_256D 643712114

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/**
 * @brief Создать контекст AEAD.
 *
 * Создаёт контекст AEAD с использованием ключа `key` в заданной области памяти.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_MGM_SIZE и выравниванием
 * #KUZNYECHIK_MGM_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 *
 * \see #KuznyechikMgm
 */
void kuznyechik_mgm_new(KuznyechikMgm dst,
                        const uint8_t key[SYMMETRIC_KEY_SIZE]);

/**
 * @brief Зашифровать сообщение с присоединёнными данными.
 *
 * Зашифровывает сообщение на месте с присоединёнными данными и копирует соответствующую
 * имитовставку в массив `tag`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст AEAD
 * @param nonce — синхропосылка как массив байт длиной #KUZNYECHIK_MGM_NONCE_SIZE. Старший
 * бит `nonce[0]` должен быть равен нулю.
 * @param aad — присоединённые данные
 * @param aad_len — длина присоединённых данных в байтах
 * @param buf — буфер, содержащий подлежащие шифрованию данные
 * @param buf_len — длина буфера (в байтах)
 * @param tag — имитовставка как массив байт длиной #KUZNYECHIK_MGM_TAG_SIZE
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #KuznyechikMgm
 */
intptr_t kuznyechik_mgm_encrypt(KuznyechikMgm cipher,
                                const uint8_t nonce[KUZNYECHIK_MGM_NONCE_SIZE],
                                const uint8_t *aad,
                                uintptr_t aad_len,
                                uint8_t *buf,
                                uintptr_t buf_len,
                                uint8_t tag[KUZNYECHIK_MGM_TAG_SIZE]);

/**
 * @brief Расшифровать сообщение с присоединёнными данными.
 *
 * Расшифровывает сообщение на месте с присоединёнными данными с использованием данной
 * имитовставки `tag`. Возвращает ошибку в случае, если имитовставка не соответствует
 * шифротексту.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст AEAD
 * @param nonce — синхропосылка как массив байт длиной #KUZNYECHIK_MGM_NONCE_SIZE
 * @param aad — присоединённые данные
 * @param aad_len — длина присоединённых данных в байтах
 * @param buf — буфер, содержащий подлежащие расшифрованию данные
 * @param buf_len — длина буфера (в байтах)
 * @param tag — имитовставка как массив байт длиной #KUZNYECHIK_MGM_TAG_SIZE
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #KuznyechikMgm
 */
intptr_t kuznyechik_mgm_decrypt(KuznyechikMgm cipher,
                                const uint8_t nonce[KUZNYECHIK_MGM_NONCE_SIZE],
                                const uint8_t *aad,
                                uintptr_t aad_len,
                                uint8_t *buf,
                                uintptr_t buf_len,
                                const uint8_t tag[KUZNYECHIK_MGM_TAG_SIZE]);

/**
 * @brief Уничтожить контекст шифра
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_mgm_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит к
 * неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #KuznyechikMgm
 */
void kuznyechik_mgm_drop(KuznyechikMgm cipher);

/**
 * @brief Создать контекст блочного шифра.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_CBC_SIZE и выравниванием
 * #KUZNYECHIK_CBC_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #KUZNYECHIK_CBC_NONCE_SIZE байт
 *
 * \see #KuznyechikCbc
 */
void kuznyechik_cbc_new(KuznyechikCbc dst,
                        const uint8_t key[SYMMETRIC_KEY_SIZE],
                        const uint8_t nonce[KUZNYECHIK_CBC_NONCE_SIZE]);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #KuznyechikCbc. Блоки шифруются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность байт длиной
 * #KUZNYECHIK_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #KuznyechikCbc
 */
void kuznyechik_cbc_encrypt_blocks(KuznyechikCbc cipher,
                                   uint8_t blocks[KUZNYECHIK_BLOCK_SIZE],
                                   uintptr_t blocks_len);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #KuznyechikCbc. Блоки расшифровываются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность байт
 * длиной #KUZNYECHIK_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #KuznyechikCbc
 */
void kuznyechik_cbc_decrypt_blocks(KuznyechikCbc cipher,
                                   uint8_t blocks[KUZNYECHIK_BLOCK_SIZE],
                                   uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение на месте с помощью заданного контекста #KuznyechikCbc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 * @param pos — индекс байта, с которого начинается дополнение в буфере `buf`
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки
 *
 * \see #KuznyechikCbc
 */
intptr_t kuznyechik_cbc_encrypt_pad(KuznyechikCbc cipher,
                                    uint8_t *buf,
                                    uintptr_t buf_len,
                                    uintptr_t pos);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение на месте с помощью заданного контекста #KuznyechikCbc. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #KUZNYECHIK_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * @returns Длину сообщения, в случае успеха или -1, в случае ошибки
 *
 * \see #KuznyechikCbc
 */
intptr_t kuznyechik_cbc_decrypt_pad(KuznyechikCbc cipher,
                                    uint8_t *buf,
                                    uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_cbc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #KuznyechikCbc
 */
void kuznyechik_cbc_drop(KuznyechikCbc cipher);

/**
 * @brief Создать контекст блочного шифра.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #MAGMA_CBC_SIZE и выравниванием
 * #MAGMA_CBC_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #MAGMA_CBC_NONCE_SIZE байт
 *
 * \see #MagmaCbc
 */
void magma_cbc_new(MagmaCbc dst,
                   const uint8_t key[SYMMETRIC_KEY_SIZE],
                   const uint8_t nonce[MAGMA_CBC_NONCE_SIZE]);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #MagmaCbc. Блоки шифруются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность байт длиной
 * #MAGMA_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #MagmaCbc
 */
void magma_cbc_encrypt_blocks(MagmaCbc cipher,
                              uint8_t blocks[MAGMA_BLOCK_SIZE],
                              uintptr_t blocks_len);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #MagmaCbc. Блоки расшифровываются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность байт
 * длиной #MAGMA_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #MagmaCbc
 */
void magma_cbc_decrypt_blocks(MagmaCbc cipher,
                              uint8_t blocks[MAGMA_BLOCK_SIZE],
                              uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение на месте с помощью заданного контекста #MagmaCbc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 * @param pos — индекс байта, с которого начинается дополнение в буфере `buf`
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки
 *
 * \see #MagmaCbc
 */
intptr_t magma_cbc_encrypt_pad(MagmaCbc cipher,
                               uint8_t *buf,
                               uintptr_t buf_len,
                               uintptr_t pos);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение на месте с помощью заданного контекста #MagmaCbc. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * @returns Длину сообщения, в случае успеха или -1, в случае ошибки
 *
 * \see #MagmaCbc
 */
intptr_t magma_cbc_decrypt_pad(MagmaCbc cipher,
                               uint8_t *buf,
                               uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_cbc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #MagmaCbc
 */
void magma_cbc_drop(MagmaCbc cipher);

/**
 * @brief Создать контекст потокового шифра.
 *
 * Создаёт контекст потокового шифра в режиме гаммирования с обратной связью по шифртексту
 * с использованием ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_CFB_SIZE и выравниванием
 * #KUZNYECHIK_CFB_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #KUZNYECHIK_CFB_NONCE_SIZE байт
 *
 * \see #KuznyechikCfb
 */
void kuznyechik_cfb_new(KuznyechikCfb dst,
                        const uint8_t key[SYMMETRIC_KEY_SIZE],
                        const uint8_t nonce[KUZNYECHIK_CFB_NONCE_SIZE]);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение на месте с помощью заданного контекста #KuznyechikCfb.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * \see #KuznyechikCfb
 */
void kuznyechik_cfb_encrypt(KuznyechikCfb cipher,
                            uint8_t *buf,
                            uintptr_t buf_len);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение на месте с помощью заданного контекста #KuznyechikCfb.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * \see #KuznyechikCfb
 */
void kuznyechik_cfb_decrypt(KuznyechikCfb cipher,
                            uint8_t *buf,
                            uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_cfb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #KuznyechikCfb
 */
void kuznyechik_cfb_drop(KuznyechikCfb cipher);

/**
 * @brief Создать контекст потокового шифра.
 *
 * Создаёт контекст потокового шифра в режиме гаммирования с обратной связью по шифртексту
 * с использованием ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #MAGMA_CFB_SIZE и выравниванием
 * #MAGMA_CFB_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #MAGMA_CFB_NONCE_SIZE байт
 *
 * \see #MagmaCfb
 */
void magma_cfb_new(MagmaCfb dst,
                   const uint8_t key[SYMMETRIC_KEY_SIZE],
                   const uint8_t nonce[MAGMA_CFB_NONCE_SIZE]);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение на месте с помощью заданного контекста #MagmaCfb.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * \see #MagmaCfb
 */
void magma_cfb_encrypt(MagmaCfb cipher,
                       uint8_t *buf,
                       uintptr_t buf_len);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение на месте с помощью заданного контекста #MagmaCfb.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * \see #MagmaCfb
 */
void magma_cfb_decrypt(MagmaCfb cipher,
                       uint8_t *buf,
                       uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_cfb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #MagmaCfb
 */
void magma_cfb_drop(MagmaCfb cipher);

/**
 * @brief Создать контекст блочного шифра.
 *
 * Создаёт новый контекст блочного шифра.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_SIZE и выравниванием
 * #KUZNYECHIK_ALIGNMENT
 * @param key - ненулевой указатель на ключ длиной #SYMMETRIC_KEY_SIZE байт.
 *
 * \see #Kuznyechik
 */
void kuznyechik_new(Kuznyechik dst,
                    const uint8_t key[SYMMETRIC_KEY_SIZE]);

/**
 * @brief Зашифровать блок данных.
 *
 * Шифрует блок с помощью заданного контекста #Kuznyechik. Блок шифруется на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param block - ненулевой указатель на блок длиной #KUZNYECHIK_BLOCK_SIZE байт.
 *
 * \see #Kuznyechik
 */
void kuznyechik_encrypt_block(Kuznyechik cipher,
                              uint8_t block[KUZNYECHIK_BLOCK_SIZE]);

/**
 * @brief Расшифровать блок данных.
 *
 * Расшифровывает блок с помощью заданного контекста #Kuznyechik. Блок расшифровывается на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param block - ненулевой указатель на блок длиной #KUZNYECHIK_BLOCK_SIZE байт.
 *
 * \see #Kuznyechik
 */
void kuznyechik_decrypt_block(Kuznyechik cipher,
                              uint8_t block[KUZNYECHIK_BLOCK_SIZE]);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует массив блоков с помощью заданного контекста #Kuznyechik. Блоки шифруются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param blocks - ненулевой указатель на массив блоков, каждый блок длиной #KUZNYECHIK_BLOCK_SIZE байт.
 * @param len - длина массива блоков.
 *
 * \see #Kuznyechik
 */
void kuznyechik_encrypt_blocks(Kuznyechik cipher,
                               uint8_t blocks[KUZNYECHIK_BLOCK_SIZE],
                               uintptr_t len);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает массив блоков с помощью заданного контекста #Kuznyechik. Блоки расшифровываются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param blocks - ненулевой указатель на массив блоков, каждый блок длиной #KUZNYECHIK_BLOCK_SIZE байт.
 * @param len - длина массива блоков.
 *
 * \see #Kuznyechik
 */
void kuznyechik_decrypt_blocks(Kuznyechik cipher,
                               uint8_t blocks[KUZNYECHIK_BLOCK_SIZE],
                               uintptr_t len);

/**
 * @brief Уничтожить контекст.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_new или скопирован из существующего.
 * После вызова этой функции работать с этим контекстом нельзя, так как он перестает быть корректным.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 *
 * \see #Kuznyechik
 */
void kuznyechik_drop(Kuznyechik cipher);

/**
 * @brief Создать контекст блочного шифра.
 *
 * Создает новый контекст блочного шифра.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #MAGMA_SIZE и выравниванием
 * #MAGMA_ALIGNMENT
 * @param key - ненулевой указатель на ключ длиной #SYMMETRIC_KEY_SIZE байт.
 *
 * \see #Magma
 */
void magma_new(Magma dst,
               const uint8_t key[SYMMETRIC_KEY_SIZE]);

/**
 * @brief Зашифровать блок данных.
 *
 * Шифрует блок с помощью заданного контекста #Magma. Блок шифруется на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param block - ненулевой указатель на блок длиной #MAGMA_BLOCK_SIZE байт.
 *
 * \see #Magma
 */
void magma_encrypt_block(Magma cipher,
                         uint8_t block[MAGMA_BLOCK_SIZE]);

/**
 * @brief Расшифровать блок данных.
 *
 * Расшифровывает блок с помощью заданного контекста #Magma. Блок расшифровывается на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param block - ненулевой указатель на блок длиной #MAGMA_BLOCK_SIZE байт.
 *
 * \see #Magma
 */
void magma_decrypt_block(Magma cipher,
                         uint8_t block[MAGMA_BLOCK_SIZE]);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует массив блоков с помощью заданного контекста #Magma. Блоки шифруются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param blocks - ненулевой указатель на массив блоков, каждый блок длиной #MAGMA_BLOCK_SIZE байт.
 * @param len - длина массива блоков.
 *
 * \see #Magma
 */
void magma_encrypt_blocks(Magma cipher,
                          uint8_t blocks[MAGMA_BLOCK_SIZE],
                          uintptr_t len);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает массив блоков с помощью заданного контекста #Magma. Блоки расшифровываются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 * @param blocks - ненулевой указатель на массив блоков, каждый блок длиной #MAGMA_BLOCK_SIZE байт.
 * @param len - длина массива блоков.
 *
 * \see #Magma
 */
void magma_decrypt_blocks(Magma cipher,
                          uint8_t blocks[MAGMA_BLOCK_SIZE],
                          uintptr_t len);

/**
 * @brief Уничтожить контекст.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_new или скопирован из существующего.
 * После вызова этой функции работать с этим контекстом нельзя, так как он перестает быть корректным.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 *
 * \see #Magma
 */
void magma_drop(Magma cipher);

/**
 * @brief Создать контекст потокового шифра.
 *
 * Создаёт контекст потокового шифра в режиме гаммирования с использованием ключа `key`
 * и синхропосылки `nonce`
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_CTR_SIZE и выравниванием
 * #KUZNYECHIK_CTR_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #KUZNYECHIK_CTR_NONCE_SIZE байт
 *
 * \see #KuznyechikCtr
 */
void kuznyechik_ctr_new(KuznyechikCtr dst,
                        const uint8_t key[SYMMETRIC_KEY_SIZE],
                        const uint8_t nonce[KUZNYECHIK_CTR_NONCE_SIZE]);

/**
 * @brief Гаммировать сообщение ключевым потоком шифра.
 *
 * Шифрует или расшифровывает сообщение на месте посредством гаммирования его ключевым потоком
 * шифра. Возвращает ошибку в случае, когда достигнут конец ключевого потока.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #KuznyechikCtr
 */
intptr_t kuznyechik_ctr_apply_keystream(KuznyechikCtr cipher,
                                        uint8_t *buf,
                                        uintptr_t buf_len);

/**
 * @brief Установить позицию ключевого потока шифра.
 *
 * Устанавливает текущую позицию ключевого потока шифра. Возвращает ошибку в случае,
 * когда требуемая позиция превышает длину ключевого потока.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param pos — новая позиция ключевого потока
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #KuznyechikCtr
 */
intptr_t kuznyechik_ctr_seek(KuznyechikCtr cipher,
                             uint64_t pos);

/**
 * @brief Получить позицию ключевого потока шифра.
 *
 * Возвращает текущую позицию ключевого потока шифра.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * @returns Текущая позиция ключевого потока
 *
 * \see #KuznyechikCtr
 */
uint64_t kuznyechik_ctr_get_pos(KuznyechikCtr cipher);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ctr_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #KuznyechikCtr
 */
void kuznyechik_ctr_drop(KuznyechikCtr cipher);

/**
 * @brief Создать контекст потокового шифра.
 *
 * Создаёт контекст потокового шифра в режиме гаммирования с использованием ключа `key`
 * и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #MAGMA_CTR_SIZE и выравниванием
 * #MAGMA_CTR_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #MAGMA_CTR_NONCE_SIZE байт
 *
 * \see #MagmaCtr
 */
void magma_ctr_new(MagmaCtr dst,
                   const uint8_t key[SYMMETRIC_KEY_SIZE],
                   const uint8_t nonce[MAGMA_CTR_NONCE_SIZE]);

/**
 * @brief Гаммировать сообщение ключевым потоком шифра.
 *
 * Шифрует или расшифровывает сообщение на месте посредством гаммирования его ключевым потоком
 * шифра. Возвращает ошибку в случае, когда достигнут конец ключевого потока.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #MagmaCtr
 */
intptr_t magma_ctr_apply_keystream(MagmaCtr cipher,
                                   uint8_t *buf,
                                   uintptr_t buf_len);

/**
 * @brief Установить позицию ключевого потока шифра.
 *
 * Устанавливает текущую позицию ключевого потока шифра. Возвращает ошибку в случае,
 * когда требуемая позиция превышает длину ключевого потока.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param pos — новая позиция ключевого потока
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #MagmaCtr
 */
intptr_t magma_ctr_seek(MagmaCtr cipher,
                        uint64_t pos);

/**
 * @brief Получить позицию ключевого потока шифра.
 *
 * Возвращает текущую позицию ключевого потока шифра.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * @returns Текущая позиция ключевого потока
 *
 * \see #MagmaCtr
 */
uint64_t magma_ctr_get_pos(MagmaCtr cipher);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ctr_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #MagmaCtr
 */
void magma_ctr_drop(MagmaCtr cipher);

/**
 * @brief Создать контекст блочного шифра.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_ECB_SIZE и выравниванием
 * #KUZNYECHIK_ECB_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 *
 * \see #KuznyechikEcb
 */
void kuznyechik_ecb_new(KuznyechikEcb dst,
                        const uint8_t key[SYMMETRIC_KEY_SIZE]);

/**
 * @brief Зашифровать массив блоков
 *
 * Шифрует блоки данных с помощью заданного контекста #KuznyechikEcb. Блоки шифруются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность байт
 * длиной #KUZNYECHIK_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #KuznyechikEcb
 */
void kuznyechik_ecb_encrypt_blocks(KuznyechikEcb cipher,
                                   uint8_t blocks[KUZNYECHIK_BLOCK_SIZE],
                                   uintptr_t blocks_len);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #KuznyechikEcb. Блоки расшифровываются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность байт
 * длиной #KUZNYECHIK_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #KuznyechikEcb
 */
void kuznyechik_ecb_decrypt_blocks(KuznyechikEcb cipher,
                                   uint8_t blocks[KUZNYECHIK_BLOCK_SIZE],
                                   uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение на месте с помощью заданного контекста #KuznyechikEcb. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 * @param pos — индекс байта, с которого начинается дополнение в буфере `buf`
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки
 *
 * \see #KuznyechikEcb
 */
intptr_t kuznyechik_ecb_encrypt_pad(KuznyechikEcb cipher,
                                    uint8_t *buf,
                                    uintptr_t buf_len,
                                    uintptr_t pos);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение на месте с помощью заданного контекста #KuznyechikEcb. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #KUZNYECHIK_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * @returns Длину сообщения, в случае успеха или -1, в случае ошибки
 *
 * \see #KuznyechikEcb
 */
intptr_t kuznyechik_ecb_decrypt_pad(KuznyechikEcb cipher,
                                    uint8_t *buf,
                                    uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ecb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #KuznyechikEcb
 */
void kuznyechik_ecb_drop(KuznyechikEcb cipher);

/**
 * @brief Создать контекст блочного шифра.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #MAGMA_ECB_SIZE и выравниванием
 * #MAGMA_ECB_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 *
 * \see #MagmaEcb
 */
void magma_ecb_new(MagmaEcb dst,
                   const uint8_t key[SYMMETRIC_KEY_SIZE]);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #MagmaEcb. Блоки шифруются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность
 * байт длиной #MAGMA_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #MagmaEcb
 */
void magma_ecb_encrypt_blocks(MagmaEcb cipher,
                              uint8_t blocks[MAGMA_BLOCK_SIZE],
                              uintptr_t blocks_len);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #MagmaEcb. Блоки расшифровываются на месте.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param blocks — ненулевой указатель на массив блоков. Блок представляет собой последовательность байт
 * длиной #MAGMA_BLOCK_SIZE.
 * @param blocks_len — длина массива блоков (в блоках)
 *
 * \see #MagmaEcb
 */
void magma_ecb_decrypt_blocks(MagmaEcb cipher,
                              uint8_t blocks[MAGMA_BLOCK_SIZE],
                              uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение на месте с помощью заданного контекста #MagmaEcb. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 * @param pos — индекс байта, с которого начинается дополнение в буфере `buf`
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки
 *
 * \see #MagmaEcb
 */
intptr_t magma_ecb_encrypt_pad(MagmaEcb cipher,
                               uint8_t *buf,
                               uintptr_t buf_len,
                               uintptr_t pos);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение на месте с помощью заданного контекста #MagmaEcb. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * @returns Длину сообщения, в случае успеха или -1, в случае ошибки
 *
 * \see #MagmaEcb
 */
intptr_t magma_ecb_decrypt_pad(MagmaEcb cipher,
                               uint8_t *buf,
                               uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ecb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #MagmaEcb
 */
void magma_ecb_drop(MagmaEcb cipher);

/**
 * @brief Создать закрытый ключ.
 *
 * Создаёт закрытый ключ для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — результирующий закрытый ключ
 * @param rand_bytes — набор чисел, сгенерированный случайным образом
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_BAD_RAND_BYTES, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_gen_private_key(Ec256 cid,
                               Ec256PrivateKey sk,
                               const uint8_t rand_bytes[EC_256_WIDTH]);

/**
 * @brief Получить открытый ключ.
 *
 * Получает открытый ключ из закрытого ключа для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param pk — результирующий открытый ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_get_public_key(Ec256 cid,
                              Ec256PrivateKey sk,
                              Ec256PublicKey pk);

/**
 * @brief Десериалировать закрытый ключ.
 *
 * Десериализует закрытый ключ из представления ГОСТ 34.10-2012 раздел 5.2/ГОСТ 34.10-2018 раздел 5.3  во внутреннее представление для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param bytes — сериализованное представление
 * @param sk — результирующий закрытый ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_PRIVATE_KEY_BYTES, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_private_key_from_bytes(Ec256 cid,
                                      const uint8_t bytes[EC_256_WIDTH],
                                      Ec256PrivateKey sk);

/**
 * @brief Сериализовать закрытый ключ.
 *
 * Сериализует закрытый ключ из внутреннего представления в представление ГОСТ 34.10-2012 раздел 5.2/ГОСТ 34.10-2018 раздел 5.3 для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param bytes — результирующее сериализованное представление
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_private_key_to_bytes(Ec256 cid,
                                    Ec256PrivateKey sk,
                                    uint8_t bytes[EC_256_WIDTH]);

/**
 * @brief Подписать хэш сообщения.
 *
 * Подписывает хэш сообщения с помощью закрытого ключа для заданной эллиптической кривой.
 * По ГОСТу хэш сообщения должен быть вычислен с помощью функции Стрибог.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param hash — хэш сообщения
 * @param rand_bytes — набор чисел, сгенерированный случайным образом
 * @param signature — результирующая подпись
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_BAD_RAND_BYTES, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_dsa_sign_hash(Ec256 cid,
                             Ec256PrivateKey sk,
                             const uint8_t hash[EC_256_WIDTH],
                             const uint8_t rand_bytes[EC_256_WIDTH],
                             uint8_t signature[EC_256_DOUBLE_WIDTH]);

/**
 * @brief Подписать сообщение.
 *
 * Подписывает сообщение с помощью закрытого ключа для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param msg — указатель на сообщение
 * @param msg_len — длина сообщения
 * @param rand_bytes — набор чисел, сгенерированный случайным образом
 * @param signature — результирующая подпись
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_BAD_RAND_BYTES, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_dsa_sign(Ec256 cid,
                        Ec256PrivateKey sk,
                        const uint8_t *msg,
                        uintptr_t msg_len,
                        const uint8_t rand_bytes[EC_256_WIDTH],
                        uint8_t signature[EC_256_DOUBLE_WIDTH]);

/**
 * @brief Десериалировать открытый ключ.
 *
 * Десериализует открытый ключ из представления ГОСТ 34.10-2012 раздел 5.2/ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param bytes — сериализованное представление
 * @param pk — результирующий открытый ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_PUBLIC_KEY_BYTES, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_public_key_from_bytes(Ec256 cid,
                                     const uint8_t bytes[EC_256_DOUBLE_WIDTH],
                                     Ec256PublicKey pk);

/**
 * @brief Сериализовать открытый ключ.
 *
 * Сериализует открытый ключ из внутреннего представления в представление ГОСТ 34.10-2012 раздел 5.2/ГОСТ 34.10-2018 раздел 5.3 для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param pk — открытый ключ
 * @param bytes — результирующее сериализованное представление
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_public_key_to_bytes(Ec256 cid,
                                   Ec256PublicKey pk,
                                   uint8_t bytes[EC_256_DOUBLE_WIDTH]);

/**
 * @brief Проверить подпись хэша сообщения.
 *
 * Проверяет подпись хэша сообщения с помощью публичной ключа для заданной эллиптической кривой.
 * По ГОСТу хэш сообщения должен быть вычислен с помощью функции Стрибог.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param pk — открытый ключ
 * @param hash — хэш сообщения
 * @param signature — подпись сообщения
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_SIGNATURE, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_dsa_verify_hash(Ec256 cid,
                               Ec256PublicKey pk,
                               const uint8_t hash[EC_256_WIDTH],
                               const uint8_t signature[EC_256_DOUBLE_WIDTH]);

/**
 * @brief Проверить подпись сообщения.
 *
 * Проверяет подпись сообщения с помощью публичной ключа для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param pk — открытый ключ
 * @param msg — сообщение
 * @param msg_len — длина сообщения
 * @param signature — подпись сообщения
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_SIGNATURE, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_dsa_verify(Ec256 cid,
                          Ec256PublicKey pk,
                          const uint8_t *msg,
                          uintptr_t msg_len,
                          const uint8_t signature[EC_256_DOUBLE_WIDTH]);

/**
 * @brief Выработать общий ключ.
 *
 * Вырабатывает 256-битный алгоритм выработки ключей обмена длиной 256 бит, используя 256-битные
 * эллиптические кривые.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается код ошибки #EC_ERR_VKO_UKM. Если используемый приватный
 * или публичный ключ статичны (т.е. используются неэфемерные ключи), то
 * рекомендуется использовать `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param cid — идентификатор кривой
 * @param sk — свой закрытый ключ
 * @param pk — чужой открытый ключ
 * @param ukm — указатель на пользовательский ключевой материал (опционально)
 * @param out — результирующий общий ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_VKO_UKM, в случае ошибки
 *
 * \see #Ec256
 */
intptr_t ec256_vko256(Ec256 cid,
                      Ec256PrivateKey sk,
                      Ec256PublicKey pk,
                      const uint8_t ukm[EC_256_WIDTH],
                      uint8_t out[EC_256_WIDTH]);

/**
 * @brief Уничтожить закрытый ключ.
 *
 * Очищает переданный ключ и обнуляет память. Ключ должен быть предварительно
 * создан с помощью функции #ec256_gen_private_key, #ec256_private_key_from_bytes
 * или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec256
 */
void ec256_private_key_drop(Ec256PrivateKey sk);

/**
 * @brief Создать закрытый ключ.
 *
 * Создаёт закрытый ключ для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — результирующий закрытый ключ
 * @param rand_bytes — набор чисел, сгенерированный случайным образом
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_BAD_RAND_BYTES, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_gen_private_key(Ec512 cid,
                               Ec512PrivateKey sk,
                               const uint8_t rand_bytes[EC_512_WIDTH]);

/**
 * @brief Получить открытый ключ.
 *
 * Получает открытый ключ из закрытого ключа для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param pk — результирующий открытый ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_get_public_key(Ec512 cid,
                              Ec512PrivateKey sk,
                              Ec512PublicKey pk);

/**
 * @brief Десериалировать закрытый ключ.
 *
 * Десериализует закрытый ключ из представления ГОСТ 34.10-2012 раздел 5.2/ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param bytes — сериализованное представление
 * @param sk — результирующий закрытый ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_PRIVATE_KEY_BYTES, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_private_key_from_bytes(Ec512 cid,
                                      const uint8_t bytes[EC_512_WIDTH],
                                      Ec512PrivateKey sk);

/**
 * @brief Сериализовать закрытый ключ.
 *
 * Сериализует закрытый ключ из внутреннего представления в представление ГОСТ 34.10-2012 раздел 5.2ГОСТ 34.10-2018 раздел 5.3 для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param bytes — результирующее сериализованное представление
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_private_key_to_bytes(Ec512 cid,
                                    Ec512PrivateKey sk,
                                    uint8_t bytes[EC_512_WIDTH]);

/**
 * @brief Подписать хэш сообщения.
 *
 * Подписывает хэш сообщения с помощью закрытого ключа для заданной эллиптической кривой.
 * По ГОСТу хэш сообщения должен быть вычислен с помощью функции Стрибог.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param hash — хэш сообщения
 * @param rand_bytes — набор чисел, сгенерированный случайным образом
 * @param signature — результирующая подпись
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_BAD_RAND_BYTES, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_dsa_sign_hash(Ec512 cid,
                             Ec512PrivateKey sk,
                             const uint8_t hash[EC_512_WIDTH],
                             const uint8_t rand_bytes[EC_512_WIDTH],
                             uint8_t signature[EC_512_DOUBLE_WIDTH]);

/**
 * @brief Подписать сообщение.
 *
 * Подписывает сообщение с помощью закрытого ключа для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param sk — закрытый ключ
 * @param msg — указатель на сообщение
 * @param msg_len — длина сообщения
 * @param rand_bytes — набор чисел, сгенерированный случайным образом
 * @param signature — результирующая подпись
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_BAD_RAND_BYTES, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_dsa_sign(Ec512 cid,
                        Ec512PrivateKey sk,
                        const uint8_t *msg,
                        uintptr_t msg_len,
                        const uint8_t rand_bytes[EC_512_WIDTH],
                        uint8_t signature[EC_512_DOUBLE_WIDTH]);

/**
 * @brief Десериалировать открытый ключ.
 *
 * Десериализует открытый ключ из представления ГОСТ 34.10-2012 раздел 5.2/ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param bytes — сериализованное представление
 * @param pk — результирующий открытый ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_PUBLIC_KEY_BYTES, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_public_key_from_bytes(Ec512 cid,
                                     const uint8_t bytes[EC_512_DOUBLE_WIDTH],
                                     Ec512PublicKey pk);

/**
 * @brief Сериализовать открытый ключ.
 *
 * Сериализует открытый ключ из внутреннего представления в представление ГОСТ 34.10-2012 раздел 5.2/ГОСТ 34.10-2018 раздел 5.3 для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param pk — открытый ключ
 * @param bytes — результирующее сериализованное представление
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_public_key_to_bytes(Ec512 cid,
                                   Ec512PublicKey pk,
                                   uint8_t bytes[EC_512_DOUBLE_WIDTH]);

/**
 * @brief Проверить подпись хэша сообщения.
 *
 * Проверяет подпись хэша сообщения с помощью публичной ключа для заданной эллиптической кривой.
 * По ГОСТу хэш сообщения должен быть вычислен с помощью функции Стрибог.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param pk — открытый ключ
 * @param hash — хэш сообщения
 * @param signature — подпись сообщения
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_SIGNATURE, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_dsa_verify_hash(Ec512 cid,
                               Ec512PublicKey pk,
                               const uint8_t hash[EC_512_WIDTH],
                               const uint8_t signature[EC_512_DOUBLE_WIDTH]);

/**
 * @brief Проверить подпись сообщения.
 *
 * Проверяет подпись сообщения с помощью публичной ключа для заданной эллиптической кривой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cid — идентификатор кривой
 * @param pk — открытый ключ
 * @param msg — сообщение
 * @param msg_len — длина сообщения
 * @param signature — подпись сообщения
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_SIGNATURE, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_dsa_verify(Ec512 cid,
                          Ec512PublicKey pk,
                          const uint8_t *msg,
                          uintptr_t msg_len,
                          const uint8_t signature[EC_512_DOUBLE_WIDTH]);

/**
 * @brief Выработать общий ключ.
 *
 * Вырабатывает алгоритм выработки ключей обмена длиной 512 бит, используя 512-битные
 * эллиптические кривые.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается код ошибки #EC_ERR_VKO_UKM. Если используемый приватный
 * или публичный ключ статичны (т.е. используются неэфемерные ключи), то
 * рекомендуется использовать `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param cid — идентификатор кривой
 * @param sk — свой закрытый ключ
 * @param pk — чужой открытый ключ
 * @param ukm — указатель на пользовательский ключевой материал (опционально)
 * @param out — результирующий общий ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_VKO_UKM, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_vko512(Ec512 cid,
                      Ec512PrivateKey sk,
                      Ec512PublicKey pk,
                      const uint8_t ukm[EC_512_WIDTH],
                      uint8_t out[EC_512_WIDTH]);

/**
 * @brief Выработать общий ключ.
 *
 * Вырабатывает алгоритм выработки ключей обмена длиной 256 бит, используя 512-битные
 * эллиптические кривые.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается код ошибки #EC_ERR_VKO_UKM. Если используемый приватный
 * или публичный ключ статичны (т.е. используются неэфемерные ключи), то
 * рекомендуется использовать `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param cid — идентификатор кривой
 * @param sk — свой закрытый ключ
 * @param pk — чужой открытый ключ
 * @param ukm — указатель на пользовательский ключевой материал (опционально)
 * @param out — результирующий общий ключ
 *
 * @returns 0, в случае успеха, #EC_ERR_CURVE_ID, #EC_ERR_VKO_UKM, в случае ошибки
 *
 * \see #Ec512
 */
intptr_t ec512_vko256(Ec512 cid,
                      Ec512PrivateKey sk,
                      Ec512PublicKey pk,
                      const uint8_t ukm[EC_512_WIDTH],
                      uint8_t out[EC_256_WIDTH]);

/**
 * @brief Уничтожить закрытый ключ.
 *
 * Очищает переданный ключ и обнуляет память. Ключ должен быть предварительно
 * создан с помощью функции #ec512_gen_private_key, #ec512_private_key_from_bytes
 * или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec512
 */
void ec512_private_key_drop(Ec512PrivateKey sk);

/**
 * @brief Создать состояние функции хэширования.
 *
 * Создаёт новый состояние функции хэширования.
 *
 * @param dst — ненулевой указатель на область памяти длиной #STREEBOG256_SIZE и выравниванием
 * #STREEBOG256_ALIGNMENT
 *
 * \see #Streebog256
 */
void streebog256_new(Streebog256 dst);

/**
 * @brief Хэшировать сообщение.
 *
 * Хэширует массив данных `msg` и копирует результат в массив `res`.
 *
 * \warning Передача нулевых или некорректных указателей приводит к неопределённому поведению.
 *
 * @param msg — ненулевой указатель на массив байт
 * @param len — длина сообщения в байтах
 * @param res — массив длиной не менее чем #STREEBOG256_RESULT_SIZE байт
 *
 * \see #Streebog256
 */
void streebog256_digest(const uint8_t *msg,
                        uintptr_t len,
                        uint8_t res[STREEBOG256_RESULT_SIZE]);

/**
 * @brief Обновить состояние функции хэширования.
 *
 * Скармливает сообщение функции хэширования, обновляя её состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 * @param msg — ненулевой указатель на массив байт
 * @param len — длина сообщения в байтах
 *
 * \see #Streebog256
 */
void streebog256_update(Streebog256 state,
                        const uint8_t *msg,
                        uintptr_t len);

/**
 * @brief Финализировать состояние функции хэширования.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 * @param res — массив длиной не менее чем #STREEBOG256_RESULT_SIZE байт
 *
 * \see #Streebog256
 */
void streebog256_finalize(Streebog256 state,
                          uint8_t res[STREEBOG256_RESULT_SIZE]);

/**
 * @brief Финализировать состояние функции хэширования и заменить его новым.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 * @param res — массив длиной не менее чем #STREEBOG256_RESULT_SIZE байт
 *
 * \see #Streebog256
 */
void streebog256_finalize_reset(Streebog256 state,
                                uint8_t res[STREEBOG256_RESULT_SIZE]);

/**
 * @brief Сбросить состояние функции хэширования.
 *
 * Заменяет состояние функции хэширования новым состоянием.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 *
 * \see #Streebog256
 */
void streebog256_reset(Streebog256 state);

/**
 * @brief Уничтожить состояние функции хэширования.
 *
 * Уничтожает состояние функции хэширования.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 *
 * \see #Streebog256
 */
void streebog256_drop(Streebog256 state);

/**
 * @brief Создать состояние функции хэширования.
 *
 * Создаёт новый состояние функции хэширования.
 *
 * @param dst — ненулевой указатель на область памяти длиной #STREEBOG512_SIZE и выравниванием
 * #STREEBOG512_ALIGNMENT
 *
 * \see #Streebog512
 */
void streebog512_new(Streebog512 dst);

/**
 * @brief Хэшировать сообщение.
 *
 * Хэширует массив данных `msg` и копирует результат в массив `res`.
 *
 * \warning Передача нулевых или некорректных указателей приводит к неопределённому поведению.
 *
 * @param msg — ненулевой указатель на массив байт
 * @param len — длина сообщения в байтах
 * @param res — массив длиной не менее чем #STREEBOG512_RESULT_SIZE байт
 *
 * \see #Streebog512
 */
void streebog512_digest(const uint8_t *msg,
                        uintptr_t len,
                        uint8_t res[STREEBOG512_RESULT_SIZE]);

/**
 * @brief Обновить состояние функции хэширования.
 *
 * Скармливает сообщение функции хэширования, обновляя её состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 * @param msg — ненулевой указатель на массив байт
 * @param len — длина сообщения в байтах
 *
 * \see #Streebog512
 */
void streebog512_update(Streebog512 state,
                        const uint8_t *msg,
                        uintptr_t len);

/**
 * @brief Финализировать состояние функции хэширования.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 * @param res — массив длиной не менее чем #STREEBOG512_RESULT_SIZE байт
 *
 * \see #Streebog512
 */
void streebog512_finalize(Streebog512 state,
                          uint8_t res[STREEBOG512_RESULT_SIZE]);

/**
 * @brief Финализировать состояние функции хэширования и заменить его новым.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 * @param res — массив длиной не менее чем #STREEBOG512_RESULT_SIZE байт
 *
 * \see #Streebog512
 */
void streebog512_finalize_reset(Streebog512 state,
                                uint8_t res[STREEBOG512_RESULT_SIZE]);

/**
 * @brief Сбросить состояние функции хэширования.
 *
 * Заменяет состояние функции хэширования новым состоянием.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 *
 * \see #Streebog512
 */
void streebog512_reset(Streebog512 state);

/**
 * @brief Уничтожить состояние функции хэширования.
 *
 * Уничтожает состояние функции хэширования.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования
 *
 * \see #Streebog512
 */
void streebog512_drop(Streebog512 state);

/**
 * @brief Создать состояние режима выработки имитовставки.
 *
 * Создаёт новый состояние режима выработки имитовставки с использованием ключа `key`.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_MAC_SIZE и выравниванием
 * #KUZNYECHIK_MAC_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 *
 * \see #KuznyechikMac
 */
void kuznyechik_mac_new(KuznyechikMac dst,
                        const uint8_t key[SYMMETRIC_KEY_SIZE]);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param msg — ненулевой указатель на массив байт
 * @param len — длина сообщения в байтах
 *
 * \see #KuznyechikMac
 */
void kuznyechik_mac_update(KuznyechikMac state,
                           const uint8_t *msg,
                           uintptr_t len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param res — массив длиной не менее чем #KUZNYECHIK_MAC_RESULT_SIZE байт
 *
 * \see #KuznyechikMac
 */
void kuznyechik_mac_finalize(KuznyechikMac state,
                             uint8_t res[KUZNYECHIK_MAC_RESULT_SIZE]);

/**
 * @brief Финализировать состояние режима выработки имитовставки и заменить его новым.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param res — массив длиной не менее чем #KUZNYECHIK_MAC_RESULT_SIZE байт
 *
 * \see #KuznyechikMac
 */
void kuznyechik_mac_finalize_reset(KuznyechikMac state,
                                   uint8_t res[KUZNYECHIK_MAC_RESULT_SIZE]);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param code — массив байт длиной #KUZNYECHIK_MAC_RESULT_SIZE
 *
 * @returns true, в случае соответствия, false в случае несоответствия
 *
 * \see #KuznyechikMac
 */
bool kuznyechik_mac_verify(KuznyechikMac state,
                           const uint8_t code[KUZNYECHIK_MAC_RESULT_SIZE]);

/**
 * @brief Сбросить состояние режима выработки имитовставки.
 *
 * Заменяет состояние режима выработки имитовставки новым состоянием.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 *
 * \see #KuznyechikMac
 */
void kuznyechik_mac_reset(KuznyechikMac state);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 *
 * \see #KuznyechikMac
 */
void kuznyechik_mac_drop(KuznyechikMac state);

/**
 * @brief Создать состояние режима выработки имитовставки.
 *
 * Создаёт новый состояние режима выработки имитовставки с использованием ключа `key`.
 *
 * @param dst — ненулевой указатель на область памяти длиной #MAGMA_MAC_SIZE и выравниванием
 * #MAGMA_MAC_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 *
 * \see #MagmaMac
 */
void magma_mac_new(MagmaMac dst,
                   const uint8_t key[SYMMETRIC_KEY_SIZE]);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param msg — ненулевой указатель на массив байт
 * @param len — длина сообщения в байтах
 *
 * \see #MagmaMac
 */
void magma_mac_update(MagmaMac state,
                      const uint8_t *msg,
                      uintptr_t len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param res — массив длиной не менее чем #MAGMA_MAC_RESULT_SIZE байт
 *
 * \see #MagmaMac
 */
void magma_mac_finalize(MagmaMac state,
                        uint8_t res[MAGMA_MAC_RESULT_SIZE]);

/**
 * @brief Финализировать состояние режима выработки имитовставки и заменить его новым.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param res — массив длиной не менее чем #MAGMA_MAC_RESULT_SIZE байт
 *
 * \see #MagmaMac
 */
void magma_mac_finalize_reset(MagmaMac state,
                              uint8_t res[MAGMA_MAC_RESULT_SIZE]);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 * @param code — массив байт длиной #MAGMA_MAC_RESULT_SIZE
 *
 * @returns true, в случае соответствия, false в случае несоответствия
 *
 * \see #MagmaMac
 */
bool magma_mac_verify(MagmaMac state,
                      const uint8_t code[MAGMA_MAC_RESULT_SIZE]);

/**
 * @brief Сбросить состояние режима выработки имитовставки.
 *
 * Заменяет состояние режима выработки имитовставки новым состоянием.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 *
 * \see #MagmaMac
 */
void magma_mac_reset(MagmaMac state);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки
 *
 * \see #MagmaMac
 */
void magma_mac_drop(MagmaMac state);

/**
 * @brief Создать контекст потокового шифра.
 *
 * Создаёт контекст потокового шифра в режиме гаммирования с обратной связью по выходу
 * с использованием ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #KUZNYECHIK_OFB_SIZE и выравниванием
 * #KUZNYECHIK_OFB_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #KUZNYECHIK_OFB_NONCE_SIZE байт
 *
 * \see #KuznyechikOfb
 */
void kuznyechik_ofb_new(KuznyechikOfb dst,
                        const uint8_t key[SYMMETRIC_KEY_SIZE],
                        const uint8_t nonce[KUZNYECHIK_OFB_NONCE_SIZE]);

/**
 * @brief Гаммировать сообщение ключевым потоком шифра.
 *
 * Шифрует или расшифровывает сообщение на месте посредством гаммирования его ключевым потоком
 * шифра.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * \see #KuznyechikOfb
 */
void kuznyechik_ofb_apply_keystream(KuznyechikOfb cipher,
                                    uint8_t *buf,
                                    uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ofb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #KuznyechikOfb
 */
void kuznyechik_ofb_drop(KuznyechikOfb cipher);

/**
 * @brief Создать контекст потокового шифра.
 *
 * Создаёт контекст потокового шифра в режиме гаммирования с обратной связью по выходу
 * с использованием ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти длиной #MAGMA_OFB_SIZE и выравниванием
 * #MAGMA_OFB_ALIGNMENT
 * @param key — ненулевой указатель на массив длинной #SYMMETRIC_KEY_SIZE байт
 * @param nonce — ненулевой указатель на массив длинной #MAGMA_OFB_NONCE_SIZE байт
 *
 * \see #MagmaOfb
 */
void magma_ofb_new(MagmaOfb dst,
                   const uint8_t key[SYMMETRIC_KEY_SIZE],
                   const uint8_t nonce[MAGMA_OFB_NONCE_SIZE]);

/**
 * @brief Гаммировать сообщение ключевым потоком шифра.
 *
 * Шифрует или расшифровывает сообщение на месте посредством гаммирования его ключевым потоком
 * шифра.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 * @param buf — ненулевой указатель на массив байт
 * @param buf_len — длина буфера (в байтах)
 *
 * \see #MagmaOfb
 */
void magma_ofb_apply_keystream(MagmaOfb cipher,
                               uint8_t *buf,
                               uintptr_t buf_len);

/**
 * @brief Уничтожить контекст шифра.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ofb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст шифра
 *
 * \see #MagmaOfb
 */
void magma_ofb_drop(MagmaOfb cipher);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif /* GOSTCRYPTO_H */
