#include "catch_amalgamated.hpp"
#include "gostcrypto.hpp"
#include "common.hpp"

template <typename CbcType, size_t BlockSize, size_t NonceSize>
void cbc_test()
{
    auto key = random_array<SYMMETRIC_KEY_SIZE>();
    auto nonce = random_array<NonceSize>();

    CbcType cbc(key, nonce);
    CbcType cbc_copy = cbc;

    std::array<uint8_t, 2048> secret = random_array<2048>();
    std::array<uint8_t, 2048> ct = secret;
    size_t pos = secret.size() / 2;

    intptr_t len = CbcType(cbc).encrypt_pad(ct.data(), ct.size(), pos);
    intptr_t plain_len = CbcType(cbc_copy).decrypt_pad(ct.data(), len);

    REQUIRE(plain_len == static_cast<intptr_t>(pos));

    std::vector<uint8_t> secret_slice { secret.begin(), secret.begin() + pos };
    std::vector<uint8_t> ct_slice { ct.begin(), ct.begin() + pos };

    REQUIRE(secret_slice == ct_slice);

    const auto blocks_num = 1;
    typedef std::array<uint8_t, BlockSize> block_t;
    std::array<block_t, blocks_num> blocks = {};
    std::array<block_t, blocks_num> expected = {};

    cbc.encrypt_blocks(reinterpret_cast<uint8_t (*)[BlockSize]>(blocks.data()), blocks.size());

    REQUIRE(blocks != expected);

    cbc_copy.decrypt_blocks(reinterpret_cast<uint8_t (*)[BlockSize]>(blocks.data()), blocks.size());

    REQUIRE(blocks == expected);
}

template <typename EcbType, size_t BlockSize>
void ecb_test()
{
    auto key = random_array<SYMMETRIC_KEY_SIZE>();

    EcbType ecb(key);
    EcbType ecb_copy = ecb;
    const auto blocks_num = 1;
    typedef std::array<uint8_t, BlockSize> block_t;
    std::array<block_t, blocks_num> blocks = {};
    std::array<block_t, blocks_num> expected = {};

    ecb.encrypt_blocks(reinterpret_cast<uint8_t (*)[BlockSize]>(blocks.data()), blocks.size());

    REQUIRE(blocks != expected);

    ecb_copy.decrypt_blocks(reinterpret_cast<uint8_t (*)[BlockSize]>(blocks.data()), blocks.size());

    REQUIRE(blocks == expected);
}


TEST_CASE("KuznyechikCbc")
{
    cbc_test<KuznyechikCbc_t, KUZNYECHIK_BLOCK_SIZE, KUZNYECHIK_CBC_NONCE_SIZE>();
}

TEST_CASE("MagmaCbc")
{
    cbc_test<MagmaCbc_t, MAGMA_BLOCK_SIZE, MAGMA_CBC_NONCE_SIZE>();
}


TEST_CASE("KuznyechikEcb")
{
    ecb_test<KuznyechikEcb_t, KUZNYECHIK_BLOCK_SIZE>();
}

TEST_CASE("MagmaEcb")
{
    ecb_test<MagmaEcb_t, MAGMA_BLOCK_SIZE>();
}
