#include <array>
#include <random>
#include <limits>
#include <iostream>
#include <type_traits>

#include "gostcrypto.hpp"

template <size_t Size>
static std::array<uint8_t, Size> random_array()
{
    static std::mt19937 gen(666);
    static std::uniform_int_distribution<> distrib(0, std::numeric_limits<uint8_t>::max());

    std::array<uint8_t, Size> array;
    for (size_t i = 0; i < array.size(); ++i)
    {
        array[i] = distrib(gen);
    }
    return array;
}

template <typename T>
static void print_as_hex(const T &container)
{
    std::cout << "Hex (size = " << container.size() << "): ";
    for (const auto &i : container)
    {
        std::cout << std::hex << "0x" << int(i) << ", ";
    }
    std::cout << std::dec << std::endl;
}

template <typename PrivateKey, size_t Width>
static PrivateKey gen_private_key(intptr_t &error_code )
{
    while (true)
    {
        const auto rand_bytes = random_array<Width>();
        PrivateKey private_key{rand_bytes, error_code};
        if (error_code == 0)
        {
            return private_key;
        }
    }
}
