#include "catch_amalgamated.hpp"
#include "gostcrypto.hpp"
#include "common.hpp"

template <typename MacType, size_t MacResult>
void mac_test()
{
    auto data = random_array<1024>();
    std::vector<uint8_t> message(1024);
    std::copy(data.begin(), data.end(), message.begin());

    auto key = random_array<SYMMETRIC_KEY_SIZE>();
    MacType mac(key);

    std::array<uint8_t, MacResult> zeroed_tag{};

    mac.update(message);
    MacType mac_copy = mac;
    auto tag = mac.finalize_reset();
    auto tag_copy = mac_copy.finalize();
    REQUIRE(tag == tag_copy);
    REQUIRE(tag != zeroed_tag);

    MacType empty_magma(key);
    auto empty_tag = empty_magma.finalize_reset();
    tag = mac.finalize();
    REQUIRE(tag != zeroed_tag);
    REQUIRE(tag == empty_tag);
    REQUIRE(true == mac.verify(tag));
}

TEST_CASE("Mac128")
{
    mac_test<KuznyechikMac_t, KUZNYECHIK_MAC_RESULT_SIZE>();
}

TEST_CASE("Mac64")
{
    mac_test<MagmaMac_t, MAGMA_MAC_RESULT_SIZE>();
}
