#include "catch_amalgamated.hpp"
#include "gostcrypto.hpp"
#include "common.hpp"

template <typename PublicKey, typename PrivateKey, size_t Width>
void test()
{
    intptr_t error_code;
    PrivateKey private_key = gen_private_key<PrivateKey, Width>(error_code);
    REQUIRE(error_code == 0);
    PublicKey public_key = private_key.get_public_key();

    SECTION("from bytes and to bytes")
    {
        SECTION("secret key")
        {
            auto bytes = private_key.to_bytes();
            PrivateKey::from_bytes(bytes, error_code);
            REQUIRE(error_code == 0);
        }

        SECTION("public key")
        {
            auto bytes = public_key.to_bytes();
            PublicKey::from_bytes(bytes, error_code);
            REQUIRE(error_code == 0);
        }
    }

    SECTION("sign and verify")
    {
        auto rand_bytes = random_array<Width>();

        std::vector<uint8_t> msg(666);
        auto signature = private_key.sign(msg, rand_bytes, error_code);
        REQUIRE(error_code == 0);

        error_code = public_key.verify(msg, signature);
        REQUIRE(error_code == 0);

        std::array<uint8_t, Width> hash {0xFF, Width};

        auto another_signature = private_key.sign_hash(hash, rand_bytes, error_code);
        REQUIRE(error_code == 0);

        error_code = public_key.verify_hash(hash, another_signature);
        REQUIRE(error_code == 0);
    }

    SECTION("expected errors")
    {
        auto bytes = public_key.to_bytes();
        bytes[0] ^= 1;
        PublicKey::from_bytes(bytes, error_code);
        REQUIRE(error_code == 0);
    }
}

TEST_CASE("Ec256::A DSA")
{
    test<Ec256PublicKey_t<Gost3410_2012_256::A>, Ec256PrivateKey_t<Gost3410_2012_256::A>, EC_256_WIDTH>();
}

TEST_CASE("Ec256::B DSA")
{
    test<Ec256PublicKey_t<Gost3410_2012_256::B>, Ec256PrivateKey_t<Gost3410_2012_256::B>, EC_256_WIDTH>();
}

TEST_CASE("Ec256::C DSA")
{
    test<Ec256PublicKey_t<Gost3410_2012_256::C>, Ec256PrivateKey_t<Gost3410_2012_256::C>, EC_256_WIDTH>();
}

TEST_CASE("Ec256::D DSA")
{
    test<Ec256PublicKey_t<Gost3410_2012_256::D>, Ec256PrivateKey_t<Gost3410_2012_256::D>, EC_256_WIDTH>();
}

TEST_CASE("Ec512::A DSA")
{
    test<Ec512PublicKey_t<Gost3410_2012_512::A>, Ec512PrivateKey_t<Gost3410_2012_512::A>, EC_512_WIDTH>();
}

TEST_CASE("Ec512::B DSA")
{
    test<Ec512PublicKey_t<Gost3410_2012_512::B>, Ec512PrivateKey_t<Gost3410_2012_512::B>, EC_512_WIDTH>();
}

TEST_CASE("Ec512::C DSA")
{
    test<Ec512PublicKey_t<Gost3410_2012_512::C>, Ec512PrivateKey_t<Gost3410_2012_512::C>, EC_512_WIDTH>();
}
